/*
  phone.cpp
  2012 Copyright (c) Seeed Technology Inc.  All right reserved.

  Author:Loovee
  2012-9-18
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "phone.h"
#include <TimerOne.h>
#if PHONESOFTSERIAL
#include <SoftwareSerial.h>
#endif
#include <Arduino.h>
#include "UI_ArduinoPhone.h"

#if PHONESOFTSERIAL
SoftwareSerial mySerial(7, 8);
#endif

/*********************************************************************************************************
** Function name: init
** Descriptions:  init serial
*********************************************************************************************************/
void phone::init()
{
#if PHONESOFTSERIAL
    mySerial.begin(19200);               // the GPRS baud rate   
#else
    Serial.begin(19200);
#endif

    Serial.println("ATH");
}

/*********************************************************************************************************
** Function name: msgSend
** Descriptions:  send sms
*********************************************************************************************************/
void phone::msgSend()
{

    Serial.print("AT+CMGF=1\r");
    delay(100);
    
    Serial.print("AT + CMGS = \"");
    Serial.print(UI.msgNum);
    Serial.println("\"");
    delay(100);
    
    Serial.println(UI.msg);
    Serial.println((char)26);
    Serial.println();

}

/*********************************************************************************************************
** Function name: makeCall
** Descriptions:  make phone call
*********************************************************************************************************/
void phone::makeCall()
{
#if PHONESOFTSERIAL
    mySerial.print("ATD + ");
#else
    Serial.print("ATD");
#endif

#if PHONESOFTSERIAL
    mySerial.println(UI.callNum);
#else
    Serial.println(UI.callNum);
#endif

#if PHONESOFTSERIAL
    mySerial.println();
#else
    Serial.println();
#endif
}

/*********************************************************************************************************
** Function name: isMsg
** Descriptions:  if get sms
*********************************************************************************************************/
bool phone::isMsg(char *str)       // check if get some message
{
    if(strCmp(8, str, "CMTI: \"SM\","))
    {
        return 1;
    }
    return 0;
}

/*********************************************************************************************************
** Function name: isMsgSendOver
** Descriptions:  if sms sended
*********************************************************************************************************/
bool phone::isMsgSendOver()
{
    // add code here
    delay(1000);
    return 1;
}

/*********************************************************************************************************
** Function name: strCmp
** Descriptions:  compare two string
*********************************************************************************************************/
bool phone::strCmp(unsigned char n, char *p1, const char *p2)
{
    char *pt = p1;
    while(*(pt)<'A' || *(pt)>'Z')
    {   
        pt++;
    }
    
    for(int i = 0; i<n; i++)
    {
        if(*(pt+i) == *(p2+i))
        {
            continue;
        }
        else 
        {
            return 0;
        }
    }
    return 1;
}

/*********************************************************************************************************
** Function name: isCall
** Descriptions:  check if some one call
*********************************************************************************************************/
bool phone::isCall(char *str)
{
    if(strCmp(4, str, "RING"))
    {
        return 1;
    }
    return 0;
}

/*********************************************************************************************************
** Function name: isCallOver
** Descriptions:  check if call over
*********************************************************************************************************/
bool phone::isCallOver(char *str)
{
    // add code here
    if(strCmp(5, str, "NO CARRIER"))
    {
        return 1;
    }
    return 0;
}

/*********************************************************************************************************
** Function name: handUpCall
** Descriptions:  handUpCall
*********************************************************************************************************/
void phone::handUpCall()   
{
#if PHONESOFTSERIAL
    mySerial.println("ATH");
#else
    Serial.println("ATH");
#endif
}

/*********************************************************************************************************
** Function name: acceptCall
** Descriptions:  acceptCall
*********************************************************************************************************/
void phone::acceptCall()
{
#if PHONESOFTSERIAL
    mySerial.println("ATA");
#else
    Serial.println("ATA");
#endif
}

phone Phone;
/*********************************************************************************************************
  END FILE
*********************************************************************************************************/